function [results]=bioSensor(uptakeChemicalInd, organismObjectiveInd, knockNum, maxW, boundedMediaFlag, maxUptakeRate, indicatorFlag, modelFlag, uMediaFlag, vMediaFlag, iterateFlag)

%constants
biomassThrsh = 0.1;
maxRxnsSize=1000;

%parameters
findMaxWFlag=0;
P=1;
% yInd - indexes of reactions that can be removed or added
% notYInd  -  indexes of reactions that can not be removed or added
% m - number of rxns
% n - number of metabolites
% notY - statistics
% reversed - rxns that are reversed
[modelU, modelV, yInd, notYInd, notY, twoSided, oneSided, reversed]=createSensorModels(boundedMediaFlag, maxRxnsSize, uptakeChemicalInd, organismObjectiveInd, maxUptakeRate, biomassThrsh, modelFlag, uMediaFlag, vMediaFlag);
ySize=length(yInd);
[metNum rxnNum] = size(modelV.S);
%part 1 - create dual for v optimization problem
%max C'v
%s.t
%[A,Ay]*[v;y]<=B
I=eye(rxnNum);
I = sparse(I);
Ssparse=sparse(modelV.S);
I1 = sparse(I(notYInd,:));
I2 = sparse(I(yInd,:));
A=[Ssparse;          %Sv=0
    -Ssparse;
    I1;   %v<=v_ub
    -I1;  %-v<=-v_lb
    I2;           %v<=y*B*v_ub
    -I2;          %-v<=-y*B*v_ub
    ];

aSizeRow=size(A,1);
vSize = rxnNum;
[Ay_v, zSize]=createAy(modelV, vSize, metNum, vSize, ySize, yInd);
[Ay_u, zSizeU]=createAy(modelU, vSize, metNum, vSize, ySize, yInd);

%so: [A,Ay]x<=B;
B_v=[sparse(2*metNum,1);
    modelV.ub(notYInd);
    -modelV.lb(notYInd);
    sparse(2*ySize,1);    ];
B_u=[sparse(2*metNum,1);
    modelU.ub(notYInd);
    -modelU.lb(notYInd);
    sparse(2*ySize,1);    ];

C_v=modelV.c;
%needs to be minimum so:
%min -C*v
%s.t
%-[A,Ay]*[v;y]>=-B
%     row - a vector of the constraints of w and z
%     var - the associated y variable
%     sense - 0 is '<=',  1 is '=='
%     comp - 0 means that the constraint is active when y equals to 1
[A_w, Ay_w ,B_w,C_w, lb_w, ub_w, wSize, wZs, row, var, sense, comp]=seperateTransposeJoin(-A, -Ay_v,-B_v,-C_v,ySize, maxW, zSize, twoSided, oneSided, indicatorFlag);

%max C_w(w  z)'
%s.t
%[A_w  Ay_w]*(w z y)  <=  B_w
awSizeRow=size(A_w,1);

%%%%%%%%     new matrices!!!   %%%%%%%%%%%%%%%%%%%%%%%%%
YsSize=length(yInd);

% Ajoined=[
%     %v constraints
%     -C_v', -P*C_w', sparse(1, vSize+ySize);                                                               %dual problem objective function = primal problem objective function
%     C_v', sparse(1, wSize+zSize+vSize+ySize);                                                 %primal problem objective function = 0
%     -C_v', sparse(1, wSize+zSize+vSize+ySize);
%     sparse(awSizeRow, vSize), A_w, sparse(awSizeRow, vSize), Ay_w;     %dual constraints
%     A, sparse(aSizeRow, wSize+zSize+vSize), Ay_v;                                        %stochiometric constraints + flux boundry constraints
%     %u constraints;
%     sparse(aSizeRow, vSize+wSize+zSize), A, Ay_u;
%     %knockout num constraint
%     zeros(1,2*vSize+wSize+zSize), -ones(1,removeYsSize), zeros(1,addYsSize) ;
%     %addition num constraints
%     zeros(1,2*vSize+wSize+zSize), zeros(1,removeYsSize), ones(1,addYsSize) ;
%     ];
% Bjoined=[
%     0;
%     0;
%     0;
%     B_w;
%     B_v;
%     B_u;
%     K1-removeYsSize;
%     K2;
%     ];
% Cjoined=[zeros(vSize+wSize+zSize,1); modelU.c;zeros(ySize,1)];

Ajoined=[
    %v constraints
    -P*C_w', sparse(1, vSize+ySize);                              %dual problem objective function >= 0
    A_w, sparse(awSizeRow, vSize), Ay_w;                  %dual constraints
    %u constraints;
    sparse(aSizeRow, wSize+zSize), A, Ay_u;
    %knockout num constraint
    zeros(1,vSize+wSize+zSize), -ones(1,YsSize) ;
    ];
Bjoined=[
    0;
    B_w;
    B_u;
    knockNum-YsSize;
    ];

Cjoined=[zeros(wSize+zSize,1); modelU.c;zeros(ySize,1)];

% lbJoined=[modelV.lb;tmpLoptKnock; modelU.lb;zeros(ySize, 1)];
% ubJoined=[modelV.ub;tmpHoptKnock; modelU.ub;ysUpperBoundOptKnock];
% IntVars=2*vSize+wSize+zSize+1:2*vSize+wSize+zSize+ySize;

lbJoined=[lb_w; modelU.lb;zeros(ySize, 1)];
ubJoined=[ub_w; modelU.ub;ones(ySize, 1)];
IntVars=vSize+wSize+zSize+1:vSize+wSize+zSize+ySize;

if(indicatorFlag)    %build logCon
    clear logCon;
    logCon(length(var)).rhs = 0;
    %     row - a vector of the constraints of w and z
    %     var - the associated y variable
    %     sense - 0 is '<=',  1 is '=='
    %     comp - 0 means that the constraint is active when y equals to 1
    for j=1:length(var)
        logCon(j).row = [row(j,:), zeros(1, vSize+ySize)];
        logCon(j).var = vSize+wSize+zSize+var(j);
        %                 logCon(j).row = [zeros(1,vSize), row(j,:), zeros(1, vSize+ySize)];
        %                 logCon(j).var = vSize+wSize+zSize+vSize+var(j);
        logCon(j).sense = sense(j);
        logCon(j).comp = comp(j);
        logCon(j).rhs = 0;
    end
end

%parameter for mip assign
x_min = []; x_max = []; f_Low = -1E7; % f_Low <= f_optimal must hold
f_opt = -141278;
nProblem = 7; % Use the same problem number as in mip_prob.m
fIP = []; % Do not use any prior knowledge
xIP = []; % Do not use any prior knowledge
setupFile = []; % Just define the Prob structure, not any permanent setup file
x_opt = []; % The optimal integer solution is not known
VarWeight = []; % No variable priorities, largest fractional part will be used
KNAPSACK = 0; % First run without the knapsack heuristic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%iterate over different solutions
i=1;
badSolFlag = 0;
knockedYind = ones(1,YsSize);
sol = inf;

while (~badSolFlag && sol > 0.05)
    yKnockVec = zeros(1,YsSize);
    yKnockVec(knockedYind) =- 1;

    Ajoined=[Ajoined;
        zeros(1, vSize+wSize+zSize), yKnockVec;
        ];

    Bjoined=[Bjoined;
        -1;
        ];

    %solve milp
    Prob=mipAssign(-Cjoined, Ajoined, [], Bjoined, lbJoined, ubJoined, [], 'part 2.3 MILP', ...
        setupFile, nProblem, ...
        IntVars, VarWeight, KNAPSACK, fIP, xIP, ...
        f_Low, x_min, x_max, f_opt, x_opt);
    if(indicatorFlag)
        Prob_OptKnock2.CPLEX.logcon = logCon;
    end
    Prob=setParams(Prob);
    Result=tomRun('cplex', Prob, 3);

    ysRemoved = IntVars(1:length(yInd));

    YsNotRound=Result.x_k(ysRemoved);
    Ys=round(YsNotRound);
    knockedYind=find(1.-Ys);
    knockedRxns=yInd(knockedYind);

    results(i).exitFlag=Result.ExitFlag;
    results(i).uptakeChemicalInd = modelV.uptakeChemicalInd;
    results(i).objective=-Result.f_k;
    results(i).uBiomassVal=Result.x_k(wSize+zSize+organismObjectiveInd);
    results(i).uptakeChemicalSize=-Result.x_k(wSize+zSize+uptakeChemicalInd);
    results(i).knockoutNum=knockNum;
    results(i).knockedRxns=knockedRxns;

    if(Result.ExitFlag == 0)        %a sucessful run
        results(i).knockCheckV=sensorCheck(modelV, knockedRxns);
        results(i).knockCheckU=sensorCheck(modelU, knockedRxns);
    end

    if (~iterateFlag)
        results(i).chemicalName = modelV.rxnNames(modelV.uptakeChemicalInd);
        results(i).maxW=maxW;

        % results.uBiomassVal=Result.x_k(vSize+wSize+zSize+modelV.organismObjectiveInd);
        % results.vBiomassVal=Result.x_k(modelV.organismObjectiveInd);
        % results.uptakeChemicalSize=Result.x_k(vSize+wSize+zSize+modelV.chemicalInd);

        results(i).x_k=Result.x_k;
        results(i).ySize=ySize;
        results(i).vSize=vSize;
        results(i).wSize=wSize;
        results(i).zSize=zSize;

        results(i).knockedYind=knockedYind;
        results(i).knockedYvals=YsNotRound(knockedYind);

        results(i).knockedRxnsNames = modelV.rxnNames(knockedRxns);
        %results(i).knockedVvals=Result.x_k(knockedRxns);
        results(i).knockedUvals=Result.x_k(knockedRxns+(wSize+zSize));

        results(i).modelV = modelV;
        results(i).modelU = modelU;
        results(i).Prob = Prob;
        results(i).Result = Result;
    end

    if (iterateFlag)
        i=i+1;
        badSolFlag = Result.ExitFlag;
        sol = -Result.f_k;
    else
        badSolFlag = 1;
    end
end
